==> grid_0.5-1.tar.gz: grid/src/util.c <==
#include "grid.h"

/* Get the list element named str, or return NULL. 
 * Copied from the Writing R Extensions manual (which copied it from nls) 
 */
SEXP getListElement(SEXP list, char *str)
{
  SEXP elmt = R_NilValue;
  SEXP names = getAttrib(list, R_NamesSymbol);
  int i;

  for (i = 0; i < length(list); i++)
    if(strcmp(CHAR(STRING_ELT(names, i)), str) == 0) {
      elmt = VECTOR_ELT(list, i);
      break;
    }
  return elmt;
}

void setListElement(SEXP list, char *str, SEXP value) 
{
  SEXP names = getAttrib(list, R_NamesSymbol);
  int i;

  for (i = 0; i < length(list); i++)
    if(strcmp(CHAR(STRING_ELT(names, i)), str) == 0) {
      SET_VECTOR_ELT(list, i, value);
      break;
    }
}
    
SEXP getSymbolValue(char *symbolName)
{
    SEXP t;
    t = findVar(install(symbolName), R_GlobalEnv);
    return t;
}

void setSymbolValue(char *symbolName, SEXP value)
{
    setVar(install(symbolName), value, R_GlobalEnv);
}

/* The lattice R code checks values to make sure that they are numeric
 * BUT we do not know whether the values are integer or real
 * SO we have to be careful when extracting numeric values.
 * This function assumes that x is numeric (obviously).
 */
double numeric(SEXP x, int index)
{
    if (isReal(x))
	return REAL(x)[index];
    else if (isInteger(x))
	return INTEGER(x)[index];
    return NA_REAL;
}

/***********************
 * Stuff for rectangles
 ***********************/

/* Fill a rectangle struct
 */
void rect(double x1, double x2, double x3, double x4, 
	  double y1, double y2, double y3, double y4, 
	  LRect *r)
{
    r->x1 = x1;
    r->x2 = x2;
    r->x3 = x3;
    r->x4 = x4;
    r->y1 = y1;
    r->y2 = y2;
    r->y3 = y3;
    r->y4 = y4;
}

void copyRect(LRect r1, LRect *r) 
{
    r->x1 = r1.x1;
    r->x2 = r1.x2;
    r->x3 = r1.x3;
    r->x4 = r1.x4;
    r->y1 = r1.y1;
    r->y2 = r1.y2;
    r->y3 = r1.y3;
    r->y4 = r1.y4;
}

/* Do two lines intersect ? 
 * Algorithm from Paul Bourke 
 * (http://www.swin.edu.au/astronomy/pbourke/geometry/lineline2d/index.html)
 */
int linesIntersect(double x1, double x2, double x3, double x4,
		   double y1, double y2, double y3, double y4)
{
    double result = 0;
    double denom = (y4 - y3)*(x2 - x1) - (x4 - x3)*(y2 - y1);
    double ua = ((x4 - x3)*(y1 - y3) - (y4 - y3)*(x1 - x3));
    /* If the lines are parallel ...
     */
    if (denom == 0) {
	/* If the lines are coincident ...
	 */
	if (ua == 0) { 
	    /* If the lines are vertical ...
	     */
	    if (x1 == x2) {
		/* Compare y-values
		 */
		if (!((y1 < y3 && fmax2(y1, y2) < fmin2(y3, y4)) ||
		      (y3 < y1 && fmax2(y3, y4) < fmin2(y1, y2))))
		    result = 1;
	    } else {
		/* Compare x-values
		 */
		if (!((x1 < x3 && fmax2(x1, x2) < fmin2(x3, x4)) ||
		      (x3 < x1 && fmax2(x3, x4) < fmin2(x1, x2))))
		    result = 1;
	    }
	}
    }
    /* ... otherwise, calculate where the lines intersect ...
     */
    else {
	double ub = ((x2 - x1)*(y1 - y3) - (y2 - y1)*(x1 - x3));
	ua = ua/denom;
	ub = ub/denom; 
	/* Check for overlap 
	 */
	if ((ua > 0 && ua < 1) && (ub > 0 && ub < 1)) 
	    result = 1;
    }
    return result;
}

int edgesIntersect(double x1, double x2, double y1, double y2,
		   LRect r)
{
    int result = 0;
    if (linesIntersect(x1, x2, r.x1, r.x2, y1, y2, r.y1, r.y2) ||
	linesIntersect(x1, x2, r.x2, r.x3, y1, y2, r.y2, r.y3) ||
	linesIntersect(x1, x2, r.x3, r.x4, y1, y2, r.y3, r.y4) ||
	linesIntersect(x1, x2, r.x4, r.x1, y1, y2, r.y4, r.y1))
	result = 1;
    return result;
}

/* Do two rects intersect ?
 * For each edge in r1, does the edge intersect with any edge in r2 ?
 * FIXME:  Should add first check for non-intersection of 
 * bounding boxes of rects (?)
 */
int intersect(LRect r1, LRect r2) 
{
    int result = 0;
    if (edgesIntersect(r1.x1, r1.x2, r1.y1, r1.y2, r2) ||
	edgesIntersect(r1.x2, r1.x3, r1.y2, r1.y3, r2) ||
	edgesIntersect(r1.x3, r1.x4, r1.y3, r1.y4, r2) ||
	edgesIntersect(r1.x4, r1.x1, r1.y4, r1.y1, r2))
	result = 1;
    return result;
}

/* FIXME:  Nicked this from Graphics.h
 * Should export it instead.
 */
#define      DEG2RAD 0.01745329251994329576

/* Calculate the bounding rectangle for a string.
 * x and y assumed to be in INCHES.
 */
void textRect(double x, double y, char *string, 
	      int font, double cex, double ps,
	      double xadj, double yadj,
	      double rot, GEDevDesc *dd, LRect *r) 
{
    /* NOTE that we must work in inches for the angles to be correct
     */
    double w = fromDeviceWidth(GEStrWidth(string, font, cex, ps, dd),
			       GE_INCHES, dd);
    double h = fromDeviceHeight(GEStrHeight(string, font, cex, ps, dd),
				GE_INCHES, dd);
    double x1, x2, x3, x4, y1, y2, y3, y4;
    double rotrad = DEG2RAD*rot;
    double sinrotrad = sin(rotrad);
    double cosrotrad = cos(rotrad);
    double wxbit = xadj*w*cosrotrad;
    double hxbit = yadj*h*sinrotrad;
    double wxbit2 = (1-xadj)*w*cosrotrad;
    double hxbit2 = (1-yadj)*h*sinrotrad;
    double wybit = xadj*w*sinrotrad;
    double hybit = yadj*h*cosrotrad;
    double wybit2 = (1-xadj)*w*sinrotrad;
    double hybit2 = (1-yadj)*h*cosrotrad;
    x1 = x - wxbit + hxbit;
    y1 = y - wybit - hybit;
    x2 = x + wxbit2 + hxbit2;
    y2 = y + wybit2 - hybit2;
    x3 = x + wxbit2 - hxbit2;
    y3 = y + wybit2 + hybit2;
    x4 = x - wxbit - hxbit;
    y4 = y - wybit + hybit;
    rect(x1, x2, x3, x4, y1, y2, y3, y4, r);
    /* For debugging, the following prints out an R statement to draw the
     * bounding box
     */
    /*  Rprintf("\ngrid.lines(c(%f, %f, %f, %f, %f), c(%f, %f, %f, %f, %f), default.units=\"inches\")\n",
	x1, x2, x3, x4, x1, y1, y2, y3, y4, y1); */
}
        
/***********************
 * Stuff for making persistent graphical objects
 ***********************/

/* Will have already created an SEXP in R.  This just stores the
 * SEXP in an external reference so that I can get multiple
 * references to it.
 */
SEXP L_CreateSEXPPtr(SEXP s)
{
    /* Allocate a list of length one on the R heap
     */
    SEXP data, result;
    PROTECT(data = allocVector(VECSXP, 1));
    SET_VECTOR_ELT(data, 0, s);
    result = R_MakeExternalPtr(data, R_NilValue, data);
    UNPROTECT(1);
    return result;
}

SEXP L_GetSEXPPtr(SEXP sp)
{
    SEXP data = R_ExternalPtrAddr(sp); 
    return VECTOR_ELT(data, 0);
}

SEXP L_SetSEXPPtr(SEXP sp, SEXP s)
{
    SEXP data = R_ExternalPtrAddr(sp);
    SET_VECTOR_ELT(data, 0, s);
    return R_NilValue;
}

==> grid_0.7-1.tar.gz: grid/src/util.c <==
#include "grid.h"

/* Get the list element named str, or return NULL. 
 * Copied from the Writing R Extensions manual (which copied it from nls) 
 */
SEXP getListElement(SEXP list, char *str)
{
  SEXP elmt = R_NilValue;
  SEXP names = getAttrib(list, R_NamesSymbol);
  int i;

  for (i = 0; i < length(list); i++)
    if(strcmp(CHAR(STRING_ELT(names, i)), str) == 0) {
      elmt = VECTOR_ELT(list, i);
      break;
    }
  return elmt;
}

void setListElement(SEXP list, char *str, SEXP value) 
{
  SEXP names = getAttrib(list, R_NamesSymbol);
  int i;

  for (i = 0; i < length(list); i++)
    if(strcmp(CHAR(STRING_ELT(names, i)), str) == 0) {
      SET_VECTOR_ELT(list, i, value);
      break;
    }
}
    
SEXP getSymbolValue(char *symbolName)
{
    SEXP t;
    t = findVar(install(symbolName), R_GlobalEnv);
    return t;
}

void setSymbolValue(char *symbolName, SEXP value)
{
    setVar(install(symbolName), value, R_GlobalEnv);
}

/* The lattice R code checks values to make sure that they are numeric
 * BUT we do not know whether the values are integer or real
 * SO we have to be careful when extracting numeric values.
 * This function assumes that x is numeric (obviously).
 */
double numeric(SEXP x, int index)
{
    if (isReal(x))
	return REAL(x)[index];
    else if (isInteger(x))
	return INTEGER(x)[index];
    return NA_REAL;
}

/***********************
 * Stuff for rectangles
 ***********************/

/* Fill a rectangle struct
 */
void rect(double x1, double x2, double x3, double x4, 
	  double y1, double y2, double y3, double y4, 
	  LRect *r)
{
    r->x1 = x1;
    r->x2 = x2;
    r->x3 = x3;
    r->x4 = x4;
    r->y1 = y1;
    r->y2 = y2;
    r->y3 = y3;
    r->y4 = y4;
}

void copyRect(LRect r1, LRect *r) 
{
    r->x1 = r1.x1;
    r->x2 = r1.x2;
    r->x3 = r1.x3;
    r->x4 = r1.x4;
    r->y1 = r1.y1;
    r->y2 = r1.y2;
    r->y3 = r1.y3;
    r->y4 = r1.y4;
}

/* Do two lines intersect ? 
 * Algorithm from Paul Bourke 
 * (http://www.swin.edu.au/astronomy/pbourke/geometry/lineline2d/index.html)
 */
int linesIntersect(double x1, double x2, double x3, double x4,
		   double y1, double y2, double y3, double y4)
{
    double result = 0;
    double denom = (y4 - y3)*(x2 - x1) - (x4 - x3)*(y2 - y1);
    double ua = ((x4 - x3)*(y1 - y3) - (y4 - y3)*(x1 - x3));
    /* If the lines are parallel ...
     */
    if (denom == 0) {
	/* If the lines are coincident ...
	 */
	if (ua == 0) { 
	    /* If the lines are vertical ...
	     */
	    if (x1 == x2) {
		/* Compare y-values
		 */
		if (!((y1 < y3 && fmax2(y1, y2) < fmin2(y3, y4)) ||
		      (y3 < y1 && fmax2(y3, y4) < fmin2(y1, y2))))
		    result = 1;
	    } else {
		/* Compare x-values
		 */
		if (!((x1 < x3 && fmax2(x1, x2) < fmin2(x3, x4)) ||
		      (x3 < x1 && fmax2(x3, x4) < fmin2(x1, x2))))
		    result = 1;
	    }
	}
    }
    /* ... otherwise, calculate where the lines intersect ...
     */
    else {
	double ub = ((x2 - x1)*(y1 - y3) - (y2 - y1)*(x1 - x3));
	ua = ua/denom;
	ub = ub/denom; 
	/* Check for overlap 
	 */
	if ((ua > 0 && ua < 1) && (ub > 0 && ub < 1)) 
	    result = 1;
    }
    return result;
}

int edgesIntersect(double x1, double x2, double y1, double y2,
		   LRect r)
{
    int result = 0;
    if (linesIntersect(x1, x2, r.x1, r.x2, y1, y2, r.y1, r.y2) ||
	linesIntersect(x1, x2, r.x2, r.x3, y1, y2, r.y2, r.y3) ||
	linesIntersect(x1, x2, r.x3, r.x4, y1, y2, r.y3, r.y4) ||
	linesIntersect(x1, x2, r.x4, r.x1, y1, y2, r.y4, r.y1))
	result = 1;
    return result;
}

/* Do two rects intersect ?
 * For each edge in r1, does the edge intersect with any edge in r2 ?
 * FIXME:  Should add first check for non-intersection of 
 * bounding boxes of rects (?)
 */
int intersect(LRect r1, LRect r2) 
{
    int result = 0;
    if (edgesIntersect(r1.x1, r1.x2, r1.y1, r1.y2, r2) ||
	edgesIntersect(r1.x2, r1.x3, r1.y2, r1.y3, r2) ||
	edgesIntersect(r1.x3, r1.x4, r1.y3, r1.y4, r2) ||
	edgesIntersect(r1.x4, r1.x1, r1.y4, r1.y1, r2))
	result = 1;
    return result;
}

/* FIXME:  Nicked this from Graphics.h
 * Should export it instead.
 */
#define      DEG2RAD 0.01745329251994329576

/* Calculate the bounding rectangle for a string.
 * x and y assumed to be in INCHES.
 */
void textRect(double x, double y, SEXP text, int i,
	      char *fontfamily, int font, double lineheight,
	      double cex, double ps,
	      double xadj, double yadj,
	      double rot, GEDevDesc *dd, LRect *r) 
{
    /* NOTE that we must work in inches for the angles to be correct
     */
    double x1, x2, x3, x4, y1, y2, y3, y4;
    double w, h, rotrad, sinrotrad, cosrotrad;
    double wxbit, hxbit, wxbit2, hxbit2;
    double wybit, hybit, wybit2, hybit2;
    if (isExpression(text)) {
	SEXP expr = VECTOR_ELT(text, i % LENGTH(text));
	w = fromDeviceWidth(GEExpressionWidth(expr, font, cex, ps, dd),
			    GE_INCHES, dd);
	h = fromDeviceHeight(GEExpressionHeight(expr, font, cex, ps, dd),
			     GE_INCHES, dd);
    } else {
	char* string = CHAR(STRING_ELT(text, i % LENGTH(text)));
	w = fromDeviceWidth(GEStrWidth(string, fontfamily, font, lineheight,
				       cex, ps, dd),
			    GE_INCHES, dd);
	h = fromDeviceHeight(GEStrHeight(string, fontfamily, font, lineheight,
					 cex, ps, dd),
			     GE_INCHES, dd);
    }
    rotrad = DEG2RAD*rot;
    sinrotrad = sin(rotrad);
    cosrotrad = cos(rotrad);
    wxbit = xadj*w*cosrotrad;
    hxbit = yadj*h*sinrotrad;
    wxbit2 = (1-xadj)*w*cosrotrad;
    hxbit2 = (1-yadj)*h*sinrotrad;
    wybit = xadj*w*sinrotrad;
    hybit = yadj*h*cosrotrad;
    wybit2 = (1-xadj)*w*sinrotrad;
    hybit2 = (1-yadj)*h*cosrotrad;
    x1 = x - wxbit + hxbit;
    y1 = y - wybit - hybit;
    x2 = x + wxbit2 + hxbit2;
    y2 = y + wybit2 - hybit2;
    x3 = x + wxbit2 - hxbit2;
    y3 = y + wybit2 + hybit2;
    x4 = x - wxbit - hxbit;
    y4 = y - wybit + hybit;
    rect(x1, x2, x3, x4, y1, y2, y3, y4, r);
    /* For debugging, the following prints out an R statement to draw the
     * bounding box
     */
    /*  Rprintf("\ngrid.lines(c(%f, %f, %f, %f, %f), c(%f, %f, %f, %f, %f), default.units=\"inches\")\n",
	x1, x2, x3, x4, x1, y1, y2, y3, y4, y1); */
}
        
/***********************
 * Stuff for making persistent graphical objects
 ***********************/

/* Will have already created an SEXP in R.  This just stores the
 * SEXP in an external reference so that I can get multiple
 * references to it.
 */
SEXP L_CreateSEXPPtr(SEXP s)
{
    /* Allocate a list of length one on the R heap
     */
    SEXP data, result;
    PROTECT(data = allocVector(VECSXP, 1));
    SET_VECTOR_ELT(data, 0, s);
    result = R_MakeExternalPtr(data, R_NilValue, data);
    UNPROTECT(1);
    return result;
}

SEXP L_GetSEXPPtr(SEXP sp)
{
    SEXP data = R_ExternalPtrAddr(sp); 
    /* Check for NULL ptr
     * This can occur if, for example, a grid grob is saved
     * and then loaded.  The saved grob has its ptr null'ed
     */
    if (data == NULL)
	error("grid grob object is empty");
    return VECTOR_ELT(data, 0);
}

SEXP L_SetSEXPPtr(SEXP sp, SEXP s)
{
    SEXP data = R_ExternalPtrAddr(sp);
    /* Check for NULL ptr
     * This can occur if, for example, a grid grob is saved
     * and then loaded.  The saved grob has its ptr null'ed
     */
    if (data == NULL)
	error("grid grob object is empty");
    SET_VECTOR_ELT(data, 0, s);
    return R_NilValue;
}
